/****************************************************************************
** Meta object code from reading C++ file 'desktopicon.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.9.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kirigami2-2.2.0/src/desktopicon.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'desktopicon.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.9.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DesktopIcon_t {
    QByteArrayData data[16];
    char stringdata0[172];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DesktopIcon_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DesktopIcon_t qt_meta_stringdata_DesktopIcon = {
    {
QT_MOC_LITERAL(0, 0, 11), // "DesktopIcon"
QT_MOC_LITERAL(1, 12, 13), // "sourceChanged"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 13), // "smoothChanged"
QT_MOC_LITERAL(4, 41, 14), // "enabledChanged"
QT_MOC_LITERAL(5, 56, 13), // "activeChanged"
QT_MOC_LITERAL(6, 70, 12), // "validChanged"
QT_MOC_LITERAL(7, 83, 15), // "selectedChanged"
QT_MOC_LITERAL(8, 99, 6), // "source"
QT_MOC_LITERAL(9, 106, 6), // "smooth"
QT_MOC_LITERAL(10, 113, 13), // "implicitWidth"
QT_MOC_LITERAL(11, 127, 14), // "implicitHeight"
QT_MOC_LITERAL(12, 142, 7), // "enabled"
QT_MOC_LITERAL(13, 150, 6), // "active"
QT_MOC_LITERAL(14, 157, 5), // "valid"
QT_MOC_LITERAL(15, 163, 8) // "selected"

    },
    "DesktopIcon\0sourceChanged\0\0smoothChanged\0"
    "enabledChanged\0activeChanged\0validChanged\0"
    "selectedChanged\0source\0smooth\0"
    "implicitWidth\0implicitHeight\0enabled\0"
    "active\0valid\0selected"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DesktopIcon[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       8,   50, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x06 /* Public */,
       3,    0,   45,    2, 0x06 /* Public */,
       4,    0,   46,    2, 0x06 /* Public */,
       5,    0,   47,    2, 0x06 /* Public */,
       6,    0,   48,    2, 0x06 /* Public */,
       7,    0,   49,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       8, QMetaType::QVariant, 0x00495103,
       9, QMetaType::Bool, 0x00495103,
      10, QMetaType::Int, 0x00095401,
      11, QMetaType::Int, 0x00095401,
      12, QMetaType::Bool, 0x00495103,
      13, QMetaType::Bool, 0x00495103,
      14, QMetaType::Bool, 0x00495001,
      15, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       0,
       0,
       2,
       3,
       4,
       5,

       0        // eod
};

void DesktopIcon::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        DesktopIcon *_t = static_cast<DesktopIcon *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->sourceChanged(); break;
        case 1: _t->smoothChanged(); break;
        case 2: _t->enabledChanged(); break;
        case 3: _t->activeChanged(); break;
        case 4: _t->validChanged(); break;
        case 5: _t->selectedChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (DesktopIcon::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&DesktopIcon::sourceChanged)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (DesktopIcon::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&DesktopIcon::smoothChanged)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (DesktopIcon::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&DesktopIcon::enabledChanged)) {
                *result = 2;
                return;
            }
        }
        {
            typedef void (DesktopIcon::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&DesktopIcon::activeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            typedef void (DesktopIcon::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&DesktopIcon::validChanged)) {
                *result = 4;
                return;
            }
        }
        {
            typedef void (DesktopIcon::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&DesktopIcon::selectedChanged)) {
                *result = 5;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        DesktopIcon *_t = static_cast<DesktopIcon *>(_o);
        Q_UNUSED(_t)
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QVariant*>(_v) = _t->source(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->smooth(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->implicitWidth(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->implicitHeight(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->isEnabled(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->active(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->valid(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->selected(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        DesktopIcon *_t = static_cast<DesktopIcon *>(_o);
        Q_UNUSED(_t)
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSource(*reinterpret_cast< QVariant*>(_v)); break;
        case 1: _t->setSmooth(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setActive(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setSelected(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    Q_UNUSED(_a);
}

const QMetaObject DesktopIcon::staticMetaObject = {
    { &QQuickItem::staticMetaObject, qt_meta_stringdata_DesktopIcon.data,
      qt_meta_data_DesktopIcon,  qt_static_metacall, nullptr, nullptr}
};


const QMetaObject *DesktopIcon::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DesktopIcon::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DesktopIcon.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int DesktopIcon::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
   else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void DesktopIcon::sourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DesktopIcon::smoothChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DesktopIcon::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void DesktopIcon::activeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void DesktopIcon::validChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void DesktopIcon::selectedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
